from typing import Dict, Any, List
import logging
from pyspark.sql import SparkSession

from Enum.FileTypeEnum import FileTypeEnum

logger = logging.getLogger()


class BucketAwsInput:

    def __init__(self, name: str, session: SparkSession, params: Dict[str, Any]) -> None:
        self.name = name
        self.session = session
        self.input_path = params["path"]
        self.input_type = params["type"]
        self.separator = params["separator"]
        self.schema = params["schema"]
        self.data = None

    def get_data(self, replace: bool, replace_space_str: str) -> None:
        try:
            def replace_delimiters(line):
                line = line.replace(replace_space_str, " ")
                return line

            file_type = FileTypeEnum(self.input_type)
            if not self.input_path.startswith("s3://") and not self.input_path.startswith("s3a://"):
                raise Exception(f"Error getting descriptor from S3. Path should start with s3://")
            final_path = self.input_path
            if file_type == FileTypeEnum.CSV or file_type == FileTypeEnum.TXT:
                if replace:
                    lines_rdd = self.session.sparkContext.textFile(final_path)
                    cleaned = lines_rdd.map(replace_delimiters)
                    self.data = self.session.read.csv(cleaned, header=True, sep=self.separator)
                else:
                    self.data = self.session.read.csv(final_path, header=True, sep=self.separator)
            elif file_type == FileTypeEnum.PARQUET:
                self.data = self.session.read.parquet(final_path, header=True)
            else:
                logger.info(f"Formato de archivo no soportado: {self.input_type}")
        except Exception as e:
            logger.error(f"Error obteniendo data de insumo. método 'get_data'. {e}")


