from typing import Dict

from pyspark.sql import SparkSession

import logging

from Enum.InputTypeEnum import InputTypeEnum

logger = logging.getLogger()


def createSession(name: str, spark_jars: Dict[str, str], source_type: InputTypeEnum) -> SparkSession:
    session = None
    try:
        jars = list(spark_jars.values())
        jars = ",".join(jars)
        session = SparkSession.builder \
            .appName(name) \
            .config("spark.jars", jars) \
            .config("spark.jars.packages", "graphframes:graphframes:0.8.3-spark3.4-s_2.12") \
            .config("spark.executor.extraClassPath", jars) \
            .config("spark.driver.extraClassPath", jars) \
            .config("spark.starrocks.driver", "com.starroks.jdbc.Driver") \
            .getOrCreate()
        session._jsc.hadoopConfiguration().set("fs.s3a.impl", "org.apache.hadoop.fs.s3a.S3AFileSystem")
        if source_type == InputTypeEnum.LOCAL:
            session._jsc.hadoopConfiguration().set("fs.s3a.endpoint", "http://192.168.21.47:9000")
            session._jsc.hadoopConfiguration().set("fs.s3a.connection.ssl.enabled", "false")
            session._jsc.hadoopConfiguration().set("fs.s3a.path.style.access", "true")
            session._jsc.hadoopConfiguration().set("fs.s3a.access.key", "minioadmin")
            session._jsc.hadoopConfiguration().set("fs.s3a.secret.key", "minioadmin")
        else:
            session.conf.set("spark.hadoop.fs.s3a.aws.credentials.provider", "com.amazonaws.auth.DefaultAWSCredentialsProviderChain")
    except Exception as e:
        logger.error(f"Error creando sesion. {e}")
    finally:
        return session


def find_related_vertices(graph):
    # Obtener vértices y aristas del grafo
    vertices = graph.vertices
    edges = graph.edges

    # Diccionario para almacenar los vértices relacionados para cada vértice
    related_vertices_dict = {}

    # Función de búsqueda en profundidad (DFS)
    def dfs(vertex_id, related_vertices):
        # Agregar el vértice actual a la lista de relacionados
        related_vertices.add(vertex_id)

        # Encontrar vértices relacionados directamente al vértice actual
        direct_related = edges.filter(edges.src == vertex_id).select("dst").collect()

        # Explorar cada vértice relacionado directamente
        for row in direct_related:
            related_vertex_id = row["dst"]
            # Si el vértice relacionado no ha sido visitado, realizar DFS en él
            if related_vertex_id not in related_vertices:
                dfs(related_vertex_id, related_vertices)

    # Obtener los valores únicos de los vértices
    unique_vertices = vertices.select("id").distinct().collect()

    # Iterar sobre los vértices únicos
    for i, row in enumerate(unique_vertices):
        vertex_id = row["id"]
        # Inicializar un conjunto para almacenar vértices relacionados
        related_vertices = set()
        # Realizar DFS para encontrar todas las relaciones del vértice actual
        dfs(vertex_id, related_vertices)
        # Agregar los vértices relacionados al diccionario
        related_vertices_dict[vertex_id] = list(related_vertices)

    return related_vertices_dict
