import time
import json
from typing import Any, Dict

from prefect import flow, get_run_logger

from Pipeline.CommissionProcess import CommissionProcess
from Enum.DatabaseTypeEnum import DatabaseTypeEnum

SPARK_JARS = {
    "STARROK": "/opt/spark-jars/starrocks-spark-connector-3.2_2.12-1.1.2.jar",
    "MYSQL": "/opt/spark-jars/mysql-connector-java-8.0.30.jar"
}

STARROK_JDBC = "jdbc:mysql://192.168.1.37:9030/bcom_spark"
STARROK_FE_NODE = "192.168.1.37:8030"

REDSHIFT_JDBC = "jdbc:redshift://redshift-cluster-1.cumpswji5bs3.us-east-1.redshift.amazonaws.com:5439/dev?currentSchema=prueba_ca"

DB_TYPE = DatabaseTypeEnum.REDSHIFT


@flow()
def run_commission(config: Dict[str, Any]) -> None:
    logger = get_run_logger()
    start_time = time.time()

    commission_process = CommissionProcess(config)

    # Conexion a Spark (LocalMode, StandAlone or Clúster)
    start_init = time.time()
    commission_process.init(SPARK_JARS)
    logger.info(f"Duración de creación de sesión Spark: {time.time() - start_init}")

    # Primer task - Extraer la data - RECORDAR: SPARK ES LAZY!!!
    start_reader = time.time()
    commission_process.get_inputs(commission_process, DB_TYPE, STARROK_JDBC, STARROK_FE_NODE, REDSHIFT_JDBC)
    logger.info(f"Duración de extracción de datos desde la BD: {time.time() - start_reader}")

    # Tercer task - Obtener metas
    start_process = time.time()
    goals = commission_process.get_goals(commission_process, "VENTAS", "GOALS")

    # Quinto task - Obtener ejecutados - ¿Aplicar tmb filtro de FLAG_COMISIONABLE y ACTIVE_USER_TRAFFIC?
    executes = commission_process.get_executed(commission_process, "VENTAS", "DEVICES")

    # Sexo task - Obtener monto origen
    base = commission_process.get_source_value(commission_process, "VENTAS", "COMERCIAL_BASE")

    result = commission_process.get_commission_per_agent(commission_process, goals, executes, base)
    logger.info(f"Duración de procesamiento en memoria: {time.time() - start_process}")

    # Task de escritura
    start_load = time.time()
    _ = commission_process.write_result(commission_process, result, "REPORT_SUMMARY", STARROK_JDBC, STARROK_FE_NODE)
    logger.info(f"Duración de carga del reporte a la BD: {time.time() - start_load}")

    logger.info(f"Duración de ejecución del proceso de comision: {time.time() - start_time}")


if __name__ == "__main__":

    conf_path = "config.json"
    with open(conf_path) as f:
        conf = json.load(f)

    # Run Commission
    run_commission(conf)


